package db

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"errors"
	"fmt"
	"reflect"
	"xorm.io/xorm"
)

// BatchSelectUserVirtualAmounts 批量查询数据 TODO::和下面的方法重复了，建议采用下面的 `UserVirtualAmountFindByParams` 方法
func BatchSelectUserVirtualAmounts(Db *xorm.Engine, params map[string]interface{}) (*[]model.UserVirtualAmount, error) {
	var UserVirtualAmountData []model.UserVirtualAmount
	if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).
		Find(&UserVirtualAmountData); err != nil {
		return nil, zhios_order_relate_logx.Warn(err)
	}
	return &UserVirtualAmountData, nil
}

// UserVirtualAmountInsert 插入单条数据
func UserVirtualAmountInsert(Db *xorm.Engine, UserVirtualAmount *model.UserVirtualAmount) (int64, error) {
	_, err := Db.InsertOne(UserVirtualAmount)
	if err != nil {
		return 0, err
	}
	return UserVirtualAmount.Id, nil
}

// BatchAddUserVirtualAmounts 批量新增数据
func BatchAddUserVirtualAmounts(Db *xorm.Engine, UserVirtualAmountData []*model.UserVirtualAmount) (int64, error) {
	affected, err := Db.Insert(UserVirtualAmountData)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func GetUserVirtualAmountCount(Db *xorm.Engine) int {
	var UserVirtualAmount model.UserVirtualAmount
	session := Db.Where("")
	count, err := session.Count(&UserVirtualAmount)
	if err != nil {
		return 0
	}
	return int(count)
}

// UserVirtualAmountDelete 删除记录
func UserVirtualAmountDelete(Db *xorm.Engine, id interface{}) (int64, error) {
	if reflect.TypeOf(id).Kind() == reflect.Slice {
		return Db.In("id", id).Delete(model.UserVirtualAmount{})
	} else {
		return Db.Where("id = ?", id).Delete(model.UserVirtualAmount{})
	}
}

// UserVirtualAmountUpdate 更新记录
func UserVirtualAmountUpdate(session *xorm.Session, id interface{}, UserVirtualAmount *model.UserVirtualAmount, forceColums ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceColums != nil {
		affected, err = session.Where("id=?", id).Cols(forceColums...).Update(UserVirtualAmount)
	} else {
		affected, err = session.Where("id=?", id).Update(UserVirtualAmount)
	}
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func GetUserVirtualWalletWithSession(session *xorm.Session, uid, coinId int) (*model.UserVirtualAmount, error) {
	var UserVirtualWallet model.UserVirtualAmount
	get, err := session.Where("uid = ? AND coin_id = ?", uid, coinId).Get(&UserVirtualWallet)
	if err != nil {
		return nil, err
	}
	if get {
		return &UserVirtualWallet, nil
	} else {
		UserVirtualWallet.Amount = "0"
		UserVirtualWallet.CoinId = coinId
		UserVirtualWallet.Uid = uid
		one, err := session.InsertOne(&UserVirtualWallet)
		if err != nil || one == 0 {
			return nil, err
		}
		return &UserVirtualWallet, nil
	}
	return nil, errors.New("获取用户虚拟币钱包失败")
}

func GetUserVirtualWallet(session *xorm.Engine, uid, coinId int) (*model.UserVirtualAmount, error) {
	var UserVirtualWallet model.UserVirtualAmount
	get, err := session.Where("uid = ? AND coin_id = ?", uid, coinId).Get(&UserVirtualWallet)
	if err != nil {
		return nil, err
	}
	if get {
		return &UserVirtualWallet, nil
	} else {
		UserVirtualWallet.Amount = "0"
		UserVirtualWallet.CoinId = coinId
		UserVirtualWallet.Uid = uid
		one, err := session.InsertOne(&UserVirtualWallet)
		if err != nil || one == 0 {
			return nil, err
		}
		return &UserVirtualWallet, nil
	}
	return nil, errors.New("获取用户虚拟币钱包失败")
}

// UserVirtualAmountGetOneByParams 通过传入的参数查询数据(单条)
func UserVirtualAmountGetOneByParams(Db *xorm.Engine, params map[string]interface{}) (*model.UserVirtualAmount, error) {
	var m model.UserVirtualAmount
	var query = fmt.Sprintf("%s =?", params["key"])
	if has, err := Db.Where(query, params["value"]).Get(&m); err != nil || has == false {
		return nil, zhios_order_relate_logx.Error(err)
	}
	return &m, nil
}

// UserVirtualAmountFindByParams 通过传入的参数查询数据(多条)
func UserVirtualAmountFindByParams(Db *xorm.Engine, params map[string]interface{}) (*[]model.UserVirtualAmount, error) {
	var m []model.UserVirtualAmount
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if params["key"] == nil {
		//查询全部数据
		err := Db.Find(&m)
		if err != nil {
			return nil, zhios_order_relate_logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).Find(&m); err != nil {
				return nil, zhios_order_relate_logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Find(&m)
			if err != nil {
				return nil, zhios_order_relate_logx.Error(err)
			}
			return &m, nil
		}

	}
}

func UserVirtualAmountFindByParamsByPage(Db *xorm.Engine, params map[string]interface{}, page, pageSize int) (*[]model.UserVirtualAmount, error) {
	var m []model.UserVirtualAmount
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if page == 0 && pageSize == 0 {
		page = 1
		pageSize = 10
	}

	if params["key"] == nil {
		//查询全部数据
		err := Db.Limit(pageSize, (page-1)*pageSize).Find(&m)
		if err != nil {
			return nil, zhios_order_relate_logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).Limit(pageSize, (page-1)*pageSize).Find(&m); err != nil {
				return nil, zhios_order_relate_logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Limit(pageSize, (page-1)*pageSize).Find(&m)
			if err != nil {
				return nil, zhios_order_relate_logx.Error(err)
			}
			return &m, nil
		}

	}
}
