package model

import (
	"time"
)

type O2oMerchant struct {
	Id                    int        `json:"id" xorm:"not null pk autoincr INT(11)"`
	Pid                   int        `json:"pid" xorm:"comment('主账号id') INT(11)"`
	AvatarUrl             string     `json:"avatar_url" xorm:"comment('用户头像') VARCHAR(255)"`
	Type                  string     `json:"type" xorm:"not null comment('商家类型，可多个') VARCHAR(255)"`
	Phone                 string     `json:"phone" xorm:"not null comment('商家手机号') VARCHAR(255)"`
	Email                 string     `json:"email" xorm:"not null comment('商家邮箱') VARCHAR(255)"`
	Wechat                string     `json:"wechat" xorm:"comment('微信号') VARCHAR(255)"`
	AccountAlipay         string     `json:"account_alipay" xorm:"comment('支付宝账户') VARCHAR(50)"`
	AccountAlipayRealName string     `json:"account_alipay_real_name" xorm:"comment('支付宝持有人姓名') VARCHAR(255)"`
	Password              string     `json:"password" xorm:"not null comment('密码') VARCHAR(255)"`
	Name                  string     `json:"name" xorm:"not null comment('名称') VARCHAR(255)"`
	FunctionalAuthority   string     `json:"functional_authority" xorm:"not null comment('功能权限角色id') VARCHAR(255)"`
	State                 int        `json:"state" xorm:"default 1 comment('商家状态') TINYINT(1)"`
	Token                 string     `json:"token" xorm:"comment('token') VARCHAR(520)"`
	Amount                float64    `json:"amount" xorm:"comment('商家余额') DOUBLE(10,2)"`
	FrozenAmount          float64    `json:"frozen_amount" xorm:"comment('商家冻结余额') DOUBLE(16,4)"`
	AuthWay               int        `json:"auth_way" xorm:"default 1 comment('1：统一授权模式，2：普通授权') TINYINT(1)"`
	CreateTime            time.Time  `json:"create_time" xorm:"not null default 'CURRENT_TIMESTAMP' comment('创建时间') DATETIME"`
	UpdateTime            time.Time  `json:"update_time" xorm:"not null default 'CURRENT_TIMESTAMP' comment('更新时间') DATETIME"`
	DeletedTime           time.Time  `json:"deleted_time" xorm:"comment('删除时间') DATETIME"`
	Gender                int        `json:"gender" xorm:"default 0 comment('0女，1：男，其他为未知性别') TINYINT(1)"`
	Birthday              *time.Time `json:"birthday" xorm:"comment('生日') DATE"`
	NotificationSetting   string     `json:"notification_setting" xorm:"comment('通知设置 json格式') TEXT"`
	Info                  string     `json:"info" xorm:"comment('补充信息') LONGTEXT"`
	BelongStore           string     `json:"belong_store" xorm:"comment('归属店铺（只有员工才有）') VARCHAR(255)"`
	RealName              string     `json:"real_name" xorm:"comment('真实姓名') VARCHAR(255)"`
	IdNumber              string     `json:"Id_number" xorm:"comment('身份证号') VARCHAR(255)"`
	InviteCode            string     `json:"invite_code" xorm:"comment('邀请码（唯一）') VARCHAR(255)"`
	IsvTicket             string     `json:"isv_ticket" xorm:"comment('支付宝isv_ticket（唯一）') VARCHAR(255)"`
	PushHandId            int        `json:"push_hand_id" xorm:"comment('推手id（推送分享出去后注册的商家绑定）') INT(11)"`
	Uid                   string     `json:"uid" xorm:"comment('智莺生活用户id') VARCHAR(50)"`
	FreezeAmount          float64    `json:"freeze_amount" xorm:"comment('商家冻结余额(B2C中用户支付后，商家的钱先流去冻结余额，完成订单才会流去余额)') DOUBLE(10,2)"`
	B2cAmount             float64    `json:"b2c_amount" xorm:"comment('b2c商家余额') DOUBLE(16,4)"`
	B2cBindAlipay         string     `json:"b2c_bind_alipay" xorm:"comment('b2c绑定的支付宝账户') VARCHAR(50)"`
	B2cBindAlipayName     string     `json:"b2c_bind_alipay_name" xorm:"comment('b2c绑定的支付宝账户姓名') VARCHAR(255)"`
	AccountWx             string     `json:"account_wx" xorm:"comment('微信账号(o2o用)') VARCHAR(255)"`
	AccountWxRealName     string     `json:"account_wx_real_name" xorm:"comment('微信账号持有人姓名(o2o用)') VARCHAR(255)"`
	AccountWxOpenId       string     `json:"account_wx_open_id" xorm:"comment('微信账号持有人的OpenId(o2o用)') VARCHAR(255)"`
	B2cBindWx             string     `json:"b2c_bind_wx" xorm:"comment('b2c绑定的微信账户') VARCHAR(255)"`
	B2cBindWxName         string     `json:"b2c_bind_wx_name" xorm:"comment('b2c绑定的微信账户姓名') VARCHAR(255)"`
	Zone                  string     `json:"zone" xorm:" default '86'  comment('区号') VARCHAR(10)"`
}
