package model

import (
	"time"
)

type RegionalAgentBase struct {
	Id                              int       `json:"id" xorm:"not null pk autoincr INT(11)"`
	OpenProvinceAgent               int       `json:"open_province_agent" xorm:"not null comment('是否开启省级代理（否：0；是：1）') TINYINT(1)"`
	ProvinceSchemeId                int       `json:"province_scheme_id" xorm:"comment('省级方案id') INT(11)"`
	OpenCityAgent                   int       `json:"open_city_agent" xorm:"not null default 0 comment('是否开启市级代理（否：0；是：1）') TINYINT(1)"`
	CitySchemeId                    int       `json:"city_scheme_id" xorm:"comment('市级方案id') INT(11)"`
	OpenDistrictAgent               int       `json:"open_district_agent" xorm:"not null default 0 comment('是否开启县区级代理（否：0；是：1）') TINYINT(1)"`
	DistrictSchemeId                int       `json:"district_scheme_id" xorm:"default 0 comment('县区级方案id') INT(11)"`
	OpenBranchesAgent               int       `json:"open_branches_agent" xorm:"not null default 0 comment('是否开启网点代理（否：0；是：1）') TINYINT(1)"`
	BranchesSchemeId                int       `json:"branches_scheme_id" xorm:"default 1 comment('网点方案id') INT(11)"`
	UpgradeSequence                 int       `json:"upgrade_sequence" xorm:"not null default 0 comment('升级顺序（1逐级升级2任意级别升级）') TINYINT(1)"`
	AutoOrderBy                     int       `json:"auto_order_by" xorm:"not null default 0 comment('订单归属，网点加入方式（1定位后自动加入，2用户手动选择加入）') TINYINT(1)"`
	ScopeOfOrder                    int       `json:"scope_of_order" xorm:"default 0 comment('当选择定位后自动加入，该字段必填；订单归属范围（km单位）用处：定位后自动绑定xxkm之内离用户最近的网点') INT(11)"`
	RemindType                      int       `json:"remind_type" xorm:"default 0 comment('当选择用户手动选择加入，该字段必填；应用内用户选择网点弹窗方式（0关闭，1正常开启，2强制开启）') TINYINT(1)"`
	ScopeOfBranches                 int       `json:"scope_of_branches" xorm:"not null comment('网点展示范围（单位：km）') INT(11)"`
	PayChannel                      string    `json:"pay_channel" xorm:"not null comment('支付方式设置（[1,2,3]）1余额支付2微信支付3支付宝支付') VARCHAR(255)"`
	CreateTime                      time.Time `json:"create_time" xorm:"not null default 'CURRENT_TIMESTAMP' comment('创建时间') DATETIME"`
	UpdateTime                      time.Time `json:"update_time" xorm:"not null default 'CURRENT_TIMESTAMP' comment('更新时间') DATETIME"`
	MallAutoOrderBy                 int       `json:"mall_auto_order_by" xorm:"default 0 comment('订单归属，网点加入方式（1定位后自动加入，2用户手动选择加入 3以订单收货地址自动归属）') TINYINT(1)"`
	O2oAutoOrderBy                  int       `json:"o2o_auto_order_by" xorm:"default 0 comment('订单归属，网点加入方式（1定位后自动加入，2用户手动选择加入 4以消费商家定位自动归属）') TINYINT(1)"`
	MallScopeOfOrder                int       `json:"mall_scope_of_order" xorm:"default 0 comment('当选择定位后自动加入，该字段必填；订单归属范围（km单位）用处：定位后自动绑定xxkm之内离用户最近的网点') INT(11)"`
	MallRemindType                  int       `json:"mall_remind_type" xorm:"default 0 comment('当选择用户手动选择加入，该字段必填；应用内用户选择网点弹窗方式（0关闭，1正常开启，2强制开启）') TINYINT(1)"`
	O2oScopeOfOrder                 int       `json:"o2o_scope_of_order" xorm:"default 0 comment('当选择定位后自动加入，该字段必填；订单归属范围（km单位）用处：定位后自动绑定xxkm之内离用户最近的网点') INT(11)"`
	O2oRemindType                   int       `json:"o2o_remind_type" xorm:"default 0 comment('当选择用户手动选择加入，该字段必填；应用内用户选择网点弹窗方式（0关闭，1正常开启，2强制开启）') TINYINT(1)"`
	IsJoin                          int       `json:"is_join" xorm:"default 0 comment('是否加入，1：是，2：否， 3:关闭') TINYINT"`
	CoinRewardOpen                  int       `json:"coin_reward_open" xorm:"default 0 comment('1：启用,2：关闭') TINYINT(1)"`
	CoinSelect                      string    `json:"coin_select" xorm:"default 0 comment('选中的虚拟币') VARCHAR(255)"`
	CoinSet                         string    `json:"coin_set" xorm:"comment('虚拟币设置') TEXT"`
	UserToBeRegionSet               int       `json:"user_to_be_region_set" xorm:"default 0 comment('用户成为区域代理设置：1、一个用户可以成为多个区域代理身份，2、一个用户只能有一个区域代理身份') TINYINT(4)"`
	AutoAddRegion                   int       `json:"auto_add_region" xorm:"default 0 comment('自动加入网点设置：1、自动获取位置并加入网点 2、手动选择网点加入') TINYINT(4)"`
	CustomerSetRegion               int       `json:"customer_set_region" xorm:"default 0 comment('网点自定义设置：1、用户可以手动添加网点 2、只允许后台添加网点') TINYINT(4)"`
	SiteOfMinimumSeparationDistance string    `json:"site_of_minimum_separation_distance" xorm:"comment('网点之间最小间隔距离(单位:km)') VARCHAR(255)"`
	CoinIdList                      string    `json:"coin_id_list" xorm:"comment('') VARCHAR(255)"`
}
