package rule

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/enum"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/svc"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/cache"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"errors"
	"fmt"
	"github.com/shopspring/decimal"
	"strconv"
	"strings"
	"time"
	"xorm.io/xorm"
)

func InitForConsumeIntegral(redisAddr string) (err error) {
	if redisAddr != "" {
		cache.NewRedis(redisAddr)
	}
	_, err = cache.SelectDb(md.RedisDataBase)
	return
}

const PessimismLockKeyForConsumeIntegral = "daily_settlement_consume_integral_lock_key:%s"
const PessimismLockValueForConsumeIntegral = "running"

// DailySettlementBlockConsumeIntegral 每日结算“消费积分”
func DailySettlementBlockConsumeIntegral(engine *xorm.Engine, mid string, isTask bool) (err error, totalMap map[int]decimal.Decimal, storeTotalMap map[int]decimal.Decimal) {
	session := engine.NewSession()
	defer func() {
		session.Close()
		if err := recover(); err != nil {

			_ = zhios_order_relate_logx.Error(err)
		}
	}()
	session.Begin()
	now := time.Now()
	today := now.Format("2006-01-02")
	storeTotalMap = make(map[int]decimal.Decimal)
	//1、查找 `subsidy_base` 基础设置
	subsidyBase, err := db.SubsidyBaseGetOneByParams(session, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})

	fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>", now.Hour())
	settleTime, err := time.ParseInLocation("2006-01-02 15:04:05", today+" "+subsidyBase.SettleTime+":00", time.Local)
	if err != nil {
		session.Rollback()
		return
	}
	if isTask && (now.Hour() > 8 || now.Before(settleTime)) {
		session.Rollback()
		//TODO::结算时间 ~ 凌晨 8 点运行
		return errors.New("非运行时间"), totalMap, storeTotalMap
	}
	if isTask && zhios_order_relate_utils.InArr(now.Weekday().String(), []string{"Sunday", "Saturday"}) && subsidyBase.HolidaySettleOpen != 1 {
		session.Rollback()
		return errors.New("周末不运行"), totalMap, storeTotalMap
	}
	if err != nil {
		_ = session.Rollback()
		return err, totalMap, storeTotalMap
	}
	if isTask && subsidyBase.SettlementDate == today {
		_ = session.Rollback()
		return errors.New("今日“消费计费”已结算"), totalMap, storeTotalMap
	}
	key := fmt.Sprintf(PessimismLockKeyForConsumeIntegral, mid)
	//TODO::增加“悲观锁”防止串行
	getString, _ := cache.GetString(key)
	//if err != nil {
	//	return err
	//}
	if getString == PessimismLockValueForConsumeIntegral {
		fmt.Println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!", "上一次结算未执行完")
		return errors.New("上一次结算未执行完"), totalMap, storeTotalMap
	}
	cache.SetEx(key, PessimismLockValueForConsumeIntegral, 3600*8) //8小时

	var consumeValueTotal, experienceValueTotal, storeValueTotal = decimal.NewFromInt(0), decimal.NewFromInt(0), decimal.NewFromInt(0)
	var consumeValue, experienceValue, storeValue float64
	//1、统计当前拥有多少份消费补贴
	startAt := time.Now().Add(time.Duration(subsidyBase.ConsumptionDay) * -24 * time.Hour).Format("2006-01-02") //起始时间
	var subsidyWithUserForConsumeList []model.SubsidyWithUser
	hasConsumeTotal, err := session.Where("date <= ?", startAt).And("kind = ? and is_can_subsidy=?", 1, 1).FindAndCount(&subsidyWithUserForConsumeList)
	if err != nil {
		_ = session.Rollback()

		return
	}
	//体验
	var subsidyWithUserForExperienceList []model.SubsidyWithUser
	hasExperienceTotal, err := session.Where("date <= ?", startAt).And("kind = ? and is_can_subsidy=?", 2, 1).FindAndCount(&subsidyWithUserForExperienceList)
	if err != nil {
		_ = session.Rollback()
		return
	}
	//门店
	var subsidyWithUserForStoreList []model.SubsidyWithUser
	hasStoreTotal, err := session.Where("date <= ?", startAt).And("kind = ? and is_can_subsidy=?", 3, 1).FindAndCount(&subsidyWithUserForStoreList)
	if err != nil {
		_ = session.Rollback()
		return
	}
	totalMap = make(map[int]decimal.Decimal)
	var consumeTotalMap = make(map[int]decimal.Decimal)
	var consumeTotalCountMap = make(map[int]int)
	total := hasConsumeTotal + hasExperienceTotal
	if subsidyBase.StoreOpen == 1 {
		total += hasStoreTotal
	}
	consumeValue, err = calcNowEverydayConsumeIntegral(subsidyBase, total)
	if err != nil {
		_ = session.Rollback()
		return err, totalMap, storeTotalMap
	}
	if hasConsumeTotal > 0 {
		for _, v := range subsidyWithUserForConsumeList {
			if v.LeaveDay > 0 {
				consumeTotalMap[v.Uid] = consumeTotalMap[v.Uid].Add(decimal.NewFromFloat(consumeValue))
				totalMap[v.Uid] = totalMap[v.Uid].Add(decimal.NewFromFloat(consumeValue))
				consumeTotalCountMap[v.Uid]++
				consumeValueTotal = consumeValueTotal.Add(decimal.NewFromFloat(consumeValue))
			}
			err1 := DealUserConsumeIntegral(session, &v, consumeValue, mid)
			if err1 != nil {
				_ = session.Rollback()
				return err1, totalMap, storeTotalMap
			}
		}
	}
	//加余额流水
	if len(consumeTotalMap) > 0 {
		for k, v := range consumeTotalMap {
			err1 := DealUserConsumeAmountFlow(session, k, zhios_order_relate_utils.StrToFloat64(v.String()), mid, zhios_order_relate_utils.IntToStr(consumeTotalCountMap[k]))
			if err1 != nil {
				_ = session.Rollback()
				return err1, totalMap, storeTotalMap
			}
		}
	}
	//2、统计当前拥有多少份体验补贴

	var experienceTotalMap = make(map[int]decimal.Decimal)
	var experienceTotalCountMap = make(map[int]int)
	if hasExperienceTotal > 0 && consumeValue > 0 {
		experienceValue, err = calcNowEverydayExperienceIntegral(consumeValue)
		if err != nil {
			_ = session.Rollback()
			return err, totalMap, storeTotalMap
		}
		for _, v := range subsidyWithUserForExperienceList {
			if v.LeaveDay > 0 {
				experienceTotalCountMap[v.Uid]++
				totalMap[v.Uid] = totalMap[v.Uid].Add(decimal.NewFromFloat(experienceValue))
				experienceTotalMap[v.Uid] = experienceTotalMap[v.Uid].Add(decimal.NewFromFloat(experienceValue))
				experienceValueTotal = experienceValueTotal.Add(decimal.NewFromFloat(experienceValue))
			}
			err1 := DealUserExperienceIntegral(session, &v, experienceValue, mid)
			if err1 != nil {
				_ = session.Rollback()
				return err1, totalMap, storeTotalMap
			}
		}
	}
	//加余额流水
	if len(experienceTotalMap) > 0 {
		for k, v := range experienceTotalMap {
			err1 := DealUserExperienceAmountFlow(session, k, zhios_order_relate_utils.StrToFloat64(v.String()), mid, zhios_order_relate_utils.IntToStr(experienceTotalCountMap[k]))
			if err1 != nil {
				_ = session.Rollback()
				return err1, totalMap, storeTotalMap
			}
		}
	}

	//2、统计当前拥有多少份门店补贴
	if subsidyBase.StoreOpen == 1 {
		var storeTotalCountMap = make(map[int]int)
		if hasStoreTotal > 0 && consumeValue > 0 {
			storeValue, err = calcNowEverydayStoreIntegral(consumeValue, zhios_order_relate_utils.StrToFloat64(subsidyBase.StoreIntegral)/zhios_order_relate_utils.StrToFloat64(subsidyBase.ConsumptionIntegral))
			if err != nil {
				_ = session.Rollback()
				return err, totalMap, storeTotalMap
			}
			for _, v := range subsidyWithUserForStoreList {
				storeTotalCountMap[v.Uid]++
				storeTotalMap[v.Uid] = storeTotalMap[v.Uid].Add(decimal.NewFromFloat(storeValue))
				storeValueTotal = storeValueTotal.Add(decimal.NewFromFloat(storeValue))
				err1 := DealUserStoreIntegral(session, &v, storeValue, mid)
				if err1 != nil {
					_ = session.Rollback()
					return err1, totalMap, storeTotalMap
				}
			}
		}
		//加余额流水
		if len(storeTotalMap) > 0 {
			for k, v := range storeTotalMap {
				err1 := DealUserStoreAmountFlow(session, k, zhios_order_relate_utils.StrToFloat64(v.String()), mid, zhios_order_relate_utils.IntToStr(storeTotalCountMap[k]))
				if err1 != nil {
					_ = session.Rollback()
					return err1, totalMap, storeTotalMap
				}
			}
		}
	}

	//3、修改 subsidy_base 中 consumption_money、experience_money
	consumeTotal, _ := consumeValueTotal.Float64()
	experienceTotal, _ := experienceValueTotal.Float64()
	storeTotal, _ := storeValueTotal.Float64()
	subsidyBase.ConsumptionTotalMoney = zhios_order_relate_utils.Float64ToStr(zhios_order_relate_utils.StrToFloat64(subsidyBase.ConsumptionTotalMoney) - consumeTotal)
	subsidyBase.ExperienceTotalMoney = zhios_order_relate_utils.Float64ToStr(zhios_order_relate_utils.StrToFloat64(subsidyBase.ExperienceTotalMoney) - experienceTotal)
	subsidyBase.StoreTotalMoney = zhios_order_relate_utils.Float64ToStr(zhios_order_relate_utils.StrToFloat64(subsidyBase.StoreTotalMoney) - storeTotal)
	subsidyBase.SettlementDate = today
	updateAffected, err := db.SubsidyBaseUpdate(session, subsidyBase.Id, subsidyBase, "consumption_total_money", "store_total_money", "experience_total_money", "settlement_date")
	if err != nil {
		_ = session.Rollback()
		return
	}
	if updateAffected <= 0 {
		_ = session.Rollback()
		return errors.New("修改 subsidy_base 失败"), totalMap, storeTotalMap
	}
	t := time.Now()
	stime := time.Date(t.Year(), t.Month(), t.Day()-1, 0, 0, 0, 0, t.Location()).Format("2006-01-02")
	dayPayAmount := "0"
	for _, v := range totalMap {
		one, _ := v.Float64()
		dayPayAmount = zhios_order_relate_utils.Float64ToStrByPrec(zhios_order_relate_utils.StrToFloat64(dayPayAmount)+one, 6)
	}
	for _, v := range storeTotalMap {
		one, _ := v.Float64()
		dayPayAmount = zhios_order_relate_utils.Float64ToStrByPrec(zhios_order_relate_utils.StrToFloat64(dayPayAmount)+one, 6)
	}
	err = Total(session, stime, dayPayAmount)
	if err != nil {
		_ = session.Rollback()
		return errors.New("事务提交失败"), totalMap, storeTotalMap
	}
	//4、提交事务
	err = session.Commit()
	if err != nil {
		_ = session.Rollback()
		return errors.New("事务提交失败"), totalMap, storeTotalMap
	}

	cache.Del(key)
	fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>消费积分结束<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<")

	return nil, totalMap, storeTotalMap
}
func Total(sess *xorm.Session, date, dayPayAmount string) error {
	sql5 := "SELECT SUM(balance_amount/(amount/integral_num)) as amount FROM subsidy_with_user"
	queryNativeString, _ := db.QueryNativeStringSess(sess, sql5)
	amount := "0"
	if len(queryNativeString) > 0 {
		amount = queryNativeString[0]["amount"]
	}
	date1 := strings.ReplaceAll(date, "-", "")
	sql2 := "SELECT SUM(integral_num) as amount FROM subsidy_with_user where kind in(1,2) and is_give=0 and date='" + date + "'"
	queryNativeString2, _ := db.QueryNativeStringSess(sess, sql2)
	DayIntegral := "0"
	if len(queryNativeString2) > 0 {
		DayIntegral = queryNativeString2[0]["amount"]
	}

	all := "SELECT SUM(amount-balance_amount) as amount FROM subsidy_with_user"
	queryNativeStringAll, _ := db.QueryNativeStringSess(sess, all)
	AllAmount := "0"
	if len(queryNativeStringAll) > 0 {
		AllAmount = queryNativeStringAll[0]["amount"]
	}
	sqlAllIntegral := "SELECT SUM(integral_num) as amount FROM subsidy_with_user where kind in(1,2) and is_give=0 "
	queryNativeStringAllIntegral, _ := db.QueryNativeStringSess(sess, sqlAllIntegral)
	AllIntegral := "0"
	if len(queryNativeStringAllIntegral) > 0 {
		AllIntegral = queryNativeStringAllIntegral[0]["amount"]
	}
	sqlAllIntegralAmount := "SELECT SUM(amount) as amount FROM subsidy_with_user where kind in(1,2) and is_give=0 "
	queryNativeStringAllIntegralAmount, _ := db.QueryNativeStringSess(sess, sqlAllIntegralAmount)
	AllIntegralAmount := "0"
	if len(queryNativeStringAllIntegralAmount) > 0 {
		AllIntegralAmount = queryNativeStringAllIntegralAmount[0]["amount"]
	}
	var data model.SubsidyTotal
	sess.Where("date=?", date1).Get(&data)
	if data.Id == 0 {
		data = model.SubsidyTotal{Date: zhios_order_relate_utils.StrToInt(date1)}
		insert, err := sess.Insert(&data)
		if insert == 0 || err != nil {
			return errors.New("失败")
		}
	}
	data.DayPayAmount = dayPayAmount
	data.NowAmount = amount
	data.DayIntegral = DayIntegral
	data.AllAmount = AllAmount
	data.AllIntegral = AllIntegral
	data.AllIntegralAmount = AllIntegralAmount
	_, err := sess.Where("id=?", data.Id).Update(&data)
	if err != nil {
		return errors.New("失败")
	}
	return nil
}

//计算当前消费积分价值（公式 "((资金池的金额 * 浮动比例) / 消费补贴总数) + 基础补贴"）
func calcNowEverydayConsumeIntegral(subsidyBase *model.SubsidyBase, hasConsumeTotal int64) (value float64, err error) {
	consumptionMoney, _ := decimal.NewFromString(subsidyBase.ConsumptionTotalMoney)
	baseSubsidyMoney, _ := decimal.NewFromString(subsidyBase.BaseSubsidyMoney)
	if subsidyBase.FloatSubsidyOpen == 0 { //没有浮动补贴情况
		subsidyBase.FloatSubsidyBili = 0
	}
	floatSubsidyBili := decimal.NewFromFloat(float64(subsidyBase.FloatSubsidyBili) / 10000)
	consumeTotal := decimal.NewFromInt(hasConsumeTotal)
	value, _ = consumptionMoney.Mul(floatSubsidyBili).Div(consumeTotal).Add(baseSubsidyMoney).RoundFloor(2).Float64()
	return
}

//计算当前体验积分价值（当前消费积分价值 * 10%）
func calcNowEverydayExperienceIntegral(consumeValue float64) (value float64, err error) {
	value, _ = decimal.NewFromFloat(consumeValue * 0.1).RoundFloor(2).Float64()
	////2、通过公式计算 "((资金池的金额 * 浮动比例) / 体验补贴总数) + 基础补贴"
	//experienceMoney, _ := decimal.NewFromString(subsidyBase.ExperienceTotalMoney)
	//baseSubsidyMoney, _ := decimal.NewFromString(subsidyBase.BaseSubsidyMoney)
	//if subsidyBase.FloatSubsidyOpen == 0 { //没有浮动补贴情况
	//	subsidyBase.FloatSubsidyBili = 0
	//}
	//floatSubsidyBili := decimal.NewFromFloat(float64(subsidyBase.FloatSubsidyBili) / 10000)
	//consumeTotal := decimal.NewFromInt(hasExperienceTotal)
	//value, _ = experienceMoney.Mul(floatSubsidyBili).Div(consumeTotal).Add(baseSubsidyMoney).RoundFloor(2).Float64()
	return
}
func calcNowEverydayStoreIntegral(consumeValue, bili float64) (value float64, err error) {
	value = zhios_order_relate_utils.FloatFormat(consumeValue*bili, 2)
	////2、通过公式计算 "((资金池的金额 * 浮动比例) / 体验补贴总数) + 基础补贴"
	//experienceMoney, _ := decimal.NewFromString(subsidyBase.ExperienceTotalMoney)
	//baseSubsidyMoney, _ := decimal.NewFromString(subsidyBase.BaseSubsidyMoney)
	//if subsidyBase.FloatSubsidyOpen == 0 { //没有浮动补贴情况
	//	subsidyBase.FloatSubsidyBili = 0
	//}
	//floatSubsidyBili := decimal.NewFromFloat(float64(subsidyBase.FloatSubsidyBili) / 10000)
	//consumeTotal := decimal.NewFromInt(hasExperienceTotal)
	//value, _ = experienceMoney.Mul(floatSubsidyBili).Div(consumeTotal).Add(baseSubsidyMoney).RoundFloor(2).Float64()
	return
}

// DealUserConsumeIntegral 处理给用户发放消费补贴奖励
func DealUserConsumeIntegral(session *xorm.Session, subsidyWithUser *model.SubsidyWithUser, consumeIntegralValue float64, mid string) (err error) {
	now := time.Now()
	//1、扣除 subsidy_with_user 的 balance_amount、state
	if subsidyWithUser.Day >= subsidyWithUser.LeaveDay && subsidyWithUser.State < 2 {
		subsidyWithUser.State = 2 //补贴中
	}
	balanceAmount := subsidyWithUser.LeaveDay
	afterAmount := balanceAmount - 1
	isEnd := 0
	if afterAmount < 0 {
		isEnd = 1
	}
	if afterAmount <= 0 {
		afterAmount = 0
		subsidyWithUser.State = 3 //补贴完
	}
	balanceAmount1, _ := decimal.NewFromString(subsidyWithUser.BalanceAmount)
	afterAmount1, _ := balanceAmount1.Sub(decimal.NewFromFloat(consumeIntegralValue)).Float64()
	if afterAmount1 <= 0 {
	}
	subsidyWithUser.BalanceAmount = zhios_order_relate_utils.Float64ToStr(afterAmount1)
	subsidyWithUser.LeaveDay = afterAmount
	updateAffected, err := db.SubsidyWithUserUpdate(session, subsidyWithUser.Id, subsidyWithUser, "leave_day", "state", "balance_amount")
	if err != nil {
		return err
	}
	if updateAffected <= 0 {
		return errors.New("修改 subsidy_with_user 记录失败")
	}
	if isEnd == 1 {
		return nil
	}
	//2、新增 subsidy_with_user_flow 记录
	var subsidyWitUserFlow = &model.SubsidyWithUserFlow{
		RecordsId: subsidyWithUser.Id,
		Day:       1,
		Amount:    zhios_order_relate_utils.Float64ToStr(consumeIntegralValue),
		LeaveDay:  afterAmount,
		CreateAt:  now.Format("2006-01-02 15:04:05"),
		UpdateAt:  now.Format("2006-01-02 15:04:05"),
		Date:      now.Format("2006-01-02"),
		Uid:       subsidyWithUser.Uid,
		Kind:      subsidyWithUser.Kind,
		Month:     zhios_order_relate_utils.StrToInt(now.Format("200601")),
	}
	insertAffected, err := db.SubsidyWithUserFlowInsert(session, subsidyWitUserFlow)
	if err != nil {
		return err
	}
	if insertAffected <= 0 {
		return errors.New("新增 subsidy_with_user_flow 记录失败")
	}
	//3、加入每月统计
	if consumeIntegralValue > 0 {
		var subsidyWitUserMonth = &model.SubsidyWithUserMonth{
			Uid:           subsidyWithUser.Uid,
			ConsumeAmount: zhios_order_relate_utils.Float64ToStr(consumeIntegralValue),
			Date:          now.Format("2006-01"),
		}
		_, err1 := db.SubsidyWithMonthInsert(session, subsidyWitUserMonth)
		if err1 != nil {
			return err
		}
	}
	//4、给用户添加余额
	orderType := enum.FinUserFlowOrderActionString(md.ConsumeSubsidyOrdActionForFinUserFlow)
	var dealUserAmount = md.DealUserAmount{
		Kind:      "add",
		Mid:       mid,
		Title:     md.ConsumeSubsidyTitleForFinUserFlow,
		OrderType: orderType,
		OrdAction: md.ConsumeSubsidyOrdActionForFinUserFlow,
		OrdId:     strconv.Itoa(subsidyWithUser.Id),
		Uid:       subsidyWithUser.Uid,
		Amount:    consumeIntegralValue,
	}
	err = svc.DealUserAmountNew(session, dealUserAmount)
	if err != nil {
		return
	}
	return nil
}

// DealUserExperienceIntegral 处理给用户发放体验补贴奖励
func DealUserExperienceIntegral(session *xorm.Session, subsidyWithUser *model.SubsidyWithUser, experienceIntegralValue float64, mid string) (err error) {
	now := time.Now()
	//1、扣除 subsidy_with_user 的 balance_amount、state
	if subsidyWithUser.Day >= subsidyWithUser.LeaveDay && subsidyWithUser.State < 2 {
		subsidyWithUser.State = 2 //补贴中
	}
	balanceAmount := subsidyWithUser.LeaveDay
	afterAmount := balanceAmount - 1
	isEnd := 0
	if afterAmount < 0 {
		isEnd = 1
	}
	if afterAmount <= 0 {
		afterAmount = 0
		subsidyWithUser.State = 3 //补贴完
	}
	balanceAmount1, _ := decimal.NewFromString(subsidyWithUser.BalanceAmount)
	afterAmount1, _ := balanceAmount1.Sub(decimal.NewFromFloat(experienceIntegralValue)).Float64()
	if afterAmount1 <= 0 {
	}
	subsidyWithUser.BalanceAmount = zhios_order_relate_utils.Float64ToStr(afterAmount1)

	subsidyWithUser.LeaveDay = afterAmount
	updateAffected, err := db.SubsidyWithUserUpdate(session, subsidyWithUser.Id, subsidyWithUser, "leave_day", "state", "balance_amount")
	if err != nil {
		return err
	}
	if updateAffected <= 0 {
		return errors.New("修改 subsidy_with_user 记录失败")
	}
	if isEnd == 1 {
		return nil
	}
	//2、新增 subsidy_with_user_flow 记录
	var subsidyWitUserFlow = &model.SubsidyWithUserFlow{
		RecordsId: subsidyWithUser.Id,
		Uid:       subsidyWithUser.Uid,
		Kind:      subsidyWithUser.Kind,
		Amount:    zhios_order_relate_utils.Float64ToStr(experienceIntegralValue),
		Day:       1,
		LeaveDay:  afterAmount,
		CreateAt:  now.Format("2006-01-02 15:04:05"),
		Date:      now.Format("2006-01-02"),
		Month:     zhios_order_relate_utils.StrToInt(now.Format("200601")),
		UpdateAt:  now.Format("2006-01-02 15:04:05"),
	}
	insertAffected, err := db.SubsidyWithUserFlowInsert(session, subsidyWitUserFlow)
	if err != nil {
		return err
	}
	if insertAffected <= 0 {
		return errors.New("新增 subsidy_with_user_flow 记录失败")
	}
	//3、加入每月统计
	if experienceIntegralValue > 0 {
		var subsidyWitUserMonth = &model.SubsidyWithUserMonth{
			Uid:              subsidyWithUser.Uid,
			ExperienceAmount: zhios_order_relate_utils.Float64ToStr(experienceIntegralValue),
			Date:             now.Format("2006-01"),
		}
		_, err1 := db.SubsidyWithMonthInsert(session, subsidyWitUserMonth)
		if err1 != nil {
			return err
		}
	}
	//4、给用户添加余额
	orderType := enum.FinUserFlowOrderActionString(md.ExperienceSubsidyOrdActionForFinUserFlow)
	var dealUserAmount = md.DealUserAmount{
		Kind:      "add",
		Mid:       mid,
		Title:     md.ExperienceSubsidyTitleForFinUserFlow,
		OrderType: orderType,
		OrdAction: md.ExperienceSubsidyOrdActionForFinUserFlow,
		OrdId:     strconv.Itoa(subsidyWithUser.Id),
		Uid:       subsidyWithUser.Uid,
		Amount:    experienceIntegralValue,
	}
	err = svc.DealUserAmountNew(session, dealUserAmount)
	if err != nil {
		return
	}
	return nil

}

// DealUserExperienceIntegral 处理给用户发放体验补贴奖励
func DealUserStoreIntegral(session *xorm.Session, subsidyWithUser *model.SubsidyWithUser, experienceIntegralValue float64, mid string) (err error) {
	now := time.Now()
	//1、扣除 subsidy_with_user 的 balance_amount、state
	if subsidyWithUser.Amount == subsidyWithUser.BalanceAmount {
		subsidyWithUser.State = 2 //补贴中
	}
	balanceAmount, _ := decimal.NewFromString(subsidyWithUser.BalanceAmount)
	afterAmount, _ := balanceAmount.Sub(decimal.NewFromFloat(experienceIntegralValue)).Float64()
	if afterAmount <= 0 {
		afterAmount = 0
		experienceIntegralValue = zhios_order_relate_utils.StrToFloat64(subsidyWithUser.BalanceAmount)
		subsidyWithUser.State = 3 //补贴完
	}
	subsidyWithUser.BalanceAmount = zhios_order_relate_utils.Float64ToStr(afterAmount)
	updateAffected, err := db.SubsidyWithUserUpdate(session, subsidyWithUser.Id, subsidyWithUser, "balance_amount", "state")
	if err != nil {
		return err
	}
	if updateAffected <= 0 {
		return errors.New("修改 subsidy_with_user 记录失败")
	}

	//2、新增 subsidy_with_user_flow 记录
	var subsidyWitUserFlow = &model.SubsidyWithUserFlow{
		RecordsId:     subsidyWithUser.Id,
		Uid:           subsidyWithUser.Uid,
		Kind:          subsidyWithUser.Kind,
		Amount:        zhios_order_relate_utils.Float64ToStr(experienceIntegralValue),
		BalanceAmount: zhios_order_relate_utils.Float64ToStr(afterAmount),
		CreateAt:      now.Format("2006-01-02 15:04:05"),
		Date:          now.Format("2006-01-02"),
		Month:         zhios_order_relate_utils.StrToInt(now.Format("200601")),
		UpdateAt:      now.Format("2006-01-02 15:04:05"),
	}
	insertAffected, err := db.SubsidyWithUserFlowInsert(session, subsidyWitUserFlow)
	if err != nil {
		return err
	}
	if insertAffected <= 0 {
		return errors.New("新增 subsidy_with_user_flow 记录失败")
	}
	//3、加入每月统计
	if experienceIntegralValue > 0 {
		var subsidyWitUserMonth = &model.SubsidyWithUserMonth{
			Uid:         subsidyWithUser.Uid,
			StoreAmount: zhios_order_relate_utils.Float64ToStr(experienceIntegralValue),
			Date:        now.Format("2006-01"),
		}
		_, err1 := db.SubsidyWithMonthInsert(session, subsidyWitUserMonth)
		if err1 != nil {
			return err
		}
	}
	//4、给用户添加余额
	orderType := enum.FinUserFlowOrderActionString(md.StoreSubsidyOrdActionForFinUserFlow)
	var dealUserAmount = md.DealUserAmount{
		Kind:      "add",
		Mid:       mid,
		Title:     md.StoreSubsidyTitleForFinUserFlow,
		OrderType: orderType,
		OrdAction: md.StoreSubsidyOrdActionForFinUserFlow,
		OrdId:     strconv.Itoa(subsidyWithUser.Id),
		Uid:       subsidyWithUser.Uid,
		Amount:    experienceIntegralValue,
	}
	err = svc.DealStoreUserAmountNew(session, dealUserAmount)
	if err != nil {
		return
	}
	return nil

}

// DealUserConsumeIntegral 处理给用户发放消费补贴奖励 流水
func DealUserConsumeAmountFlow(session *xorm.Session, uid int, consumeIntegralValue float64, mid, num string) (err error) {
	//4、给用户添加余额流水
	orderType := enum.FinUserFlowOrderActionString(md.ConsumeSubsidyOrdActionForFinUserFlow)
	var dealUserAmount = md.DealUserAmount{
		Kind:      "add",
		Mid:       mid,
		Title:     md.ConsumeSubsidyTitleForFinUserFlow,
		OrderType: orderType,
		OrdAction: md.ConsumeSubsidyOrdActionForFinUserFlow,
		Uid:       uid,
		Amount:    consumeIntegralValue,
		Num:       num,
	}
	err = svc.DealUserAmountFlow(session, dealUserAmount)
	if err != nil {
		return
	}
	return nil
}

// DealUserExperienceIntegral 处理给用户发放体验补贴奖励
func DealUserExperienceAmountFlow(session *xorm.Session, uid int, experienceIntegralValue float64, mid, num string) (err error) {
	//4、给用户添加余额
	orderType := enum.FinUserFlowOrderActionString(md.ExperienceSubsidyOrdActionForFinUserFlow)
	var dealUserAmount = md.DealUserAmount{
		Kind:      "add",
		Mid:       mid,
		Title:     md.ExperienceSubsidyTitleForFinUserFlow,
		OrderType: orderType,
		OrdAction: md.ExperienceSubsidyOrdActionForFinUserFlow,
		Uid:       uid,
		Amount:    experienceIntegralValue,
		Num:       num,
	}
	err = svc.DealUserAmountFlow(session, dealUserAmount)
	if err != nil {
		return
	}
	return nil

}

// DealUserExperienceIntegral 处理给用户发放体验补贴奖励
func DealUserStoreAmountFlow(session *xorm.Session, uid int, experienceIntegralValue float64, mid, num string) (err error) {
	//4、给用户添加余额
	orderType := enum.FinUserFlowOrderActionString(md.StoreSubsidyOrdActionForFinUserFlow)
	var dealUserAmount = md.DealUserAmount{
		Kind:      "add",
		Mid:       mid,
		Title:     md.StoreSubsidyTitleForFinUserFlow,
		OrderType: orderType,
		OrdAction: md.StoreSubsidyOrdActionForFinUserFlow,
		Uid:       uid,
		Amount:    experienceIntegralValue,
		Num:       num,
	}
	err = svc.DealUserStoreAmountFlow(session, dealUserAmount)
	if err != nil {
		return
	}
	return nil

}
