package micro_applications

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule/micro_applications/enum"
	md2 "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule/micro_applications/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_model.git/src/implement"
	"code.fnuoos.com/go_rely_warehouse/zyos_model.git/src/models"
	"errors"
	"xorm.io/xorm"
)

// GetHappyOrchardUserSeedRecordData 获取用户种子记录对应数据
func GetHappyOrchardUserSeedRecordData(engine *xorm.Engine, record models.HappyOrchardUserSeedRecord) (resp md2.GetHappyOrchardUserSeedRecordDataResp, err error) {
	//1、获取基础设置
	happyOrchardBasicSettingDb := implement.NewHappyOrchardBasicSettingDb(engine)
	happyOrchardBasicSetting, err := happyOrchardBasicSettingDb.GetHappyOrchardBasicSetting()
	if err != nil {
		return
	}
	if happyOrchardBasicSetting == nil {
		err = errors.New("请先完善基础配置")
		return
	}
	if happyOrchardBasicSetting.IsOpen == int(enum.HappyOrchardBasicSettingIsOpenForFalse) {
		err = errors.New("开心农场游戏暂未开启")
		return
	}

	//2、获取种子数据
	happyOrchardSeedDb := implement.NewHappyOrchardSeedDb(engine)
	happyOrchardSeed, err := happyOrchardSeedDb.GetHappyOrchardSeed(record.SeedId)
	if err != nil {
		return
	}
	if happyOrchardSeed == nil {
		err = errors.New("未查询到相关种子记录")
		return
	}

	//3、判断当前种子状态
	if record.State == int(enum.HappyOrchardUserSeedRecordStateForPlanting) {
		//4、解析基础设置中的 "阶段自定义" 数据
		stageNameCustomData, err := AnalysisStageNameCustom(happyOrchardBasicSetting.StageNameCustom)
		if err != nil {
			return
		}
		seedState, nextSeedStateWaters := getHappyOrchardUserSeedStage(record, *happyOrchardSeed)
		resp.RecordPlantingStage = seedState
		resp.RecordPlantingStageZh = stageNameCustomData[resp.RecordPlantingStage]
		resp.RecordPlantingNextStage = resp.RecordPlantingStage + 1
		resp.RecordPlantingNextStageZh = stageNameCustomData[resp.RecordPlantingNextStage]
		resp.RecordPlantingNextStageNeedWaterNums = nextSeedStateWaters / happyOrchardBasicSetting.WateringEveryTimeWaterDroplet
		resp.RecordPlantingNextStageNeedWater = nextSeedStateWaters
	}
	resp.RecordId = record.Id
	resp.RecordName = happyOrchardSeed.Name
	resp.RecordState = record.State
	resp.RecordStateZh = enum.HappyOrchardUserSeedRecordState(record.State).String()
	resp.NowRecordWaters = record.WaterNums
	resp.TotalNeedWaters = happyOrchardSeed.WaterNums
	resp.PlantingRate = float64(record.WaterNums) / float64(happyOrchardSeed.WaterNums) * 100
	return
}

func getHappyOrchardUserSeedStage(record models.HappyOrchardUserSeedRecord, seed models.HappyOrchardSeed) (seedState int, nextSeedStateWaters int) {
	seedState = -1
	if record.WaterNums >= seed.NeedWatersNumForStage0 && record.WaterNums < seed.NeedWatersNumForStage1 {
		seedState = 0
		nextSeedStateWaters = seed.NeedWatersNumForStage1
		return
	}
	if record.WaterNums >= seed.NeedWatersNumForStage1 && record.WaterNums < seed.NeedWatersNumForStage2 {
		seedState = 1
		nextSeedStateWaters = seed.NeedWatersNumForStage2
	}
	if record.WaterNums >= seed.NeedWatersNumForStage2 && record.WaterNums < seed.NeedWatersNumForStage3 {
		seedState = 2
		nextSeedStateWaters = seed.NeedWatersNumForStage3
	}
	if record.WaterNums >= seed.NeedWatersNumForStage3 && record.WaterNums < seed.NeedWatersNumForStage4 {
		seedState = 3
		nextSeedStateWaters = seed.NeedWatersNumForStage4
	}
	if record.WaterNums >= seed.NeedWatersNumForStage4 && record.WaterNums < seed.NeedWatersNumForStage5 {
		seedState = 4
		nextSeedStateWaters = seed.NeedWatersNumForStage5
	}
	if record.WaterNums >= seed.NeedWatersNumForStage5 {
		seedState = 5
	}
	return
}

// UserReplaceSeed  用户替换种子
func UserReplaceSeed(engine *xorm.Engine, record models.HappyOrchardUserSeedRecord, toBeSeedId, uid int) (err error) {
	//1、获取基础设置
	happyOrchardBasicSettingDb := implement.NewHappyOrchardBasicSettingDb(engine)
	happyOrchardBasicSetting, err := happyOrchardBasicSettingDb.GetHappyOrchardBasicSetting()
	if err != nil {
		return
	}
	if happyOrchardBasicSetting == nil {
		err = errors.New("请先完善基础配置")
		return
	}
	if happyOrchardBasicSetting.IsOpen == int(enum.HappyOrchardBasicSettingIsOpenForFalse) {
		err = errors.New("开心农场游戏暂未开启")
		return
	}

	//2、获取 happy_orchard_user_waters 数据
	happyOrchardUserWatersDb := implement.NewHappyOrchardUserWatersDb(engine)
	happyOrchardUserWaters, err := happyOrchardUserWatersDb.GetHappyOrchardUserWaters(uid)
	if err != nil {
		return
	}
	if happyOrchardUserWaters == nil {
		err = errors.New("未查询到相关用户数据记录")
		return
	}
	if happyOrchardUserWaters.ReplaceSeedNums <= 0 {
		err = errors.New("当前用户替换种子次数已用完")
		return
	}

	//3、判断当前种子状态
	if record.State != int(enum.HappyOrchardUserSeedRecordStateForPlanting) {
		err = errors.New("当前种子状态不可替换")
		return
	}

	//4、查找当前种子&&目标种子记录
	happyOrchardSeedDb := implement.NewHappyOrchardSeedDb(engine)
	nowHappyOrchardSeed, err := happyOrchardSeedDb.GetHappyOrchardSeed(record.SeedId)
	if err != nil {
		return
	}
	if nowHappyOrchardSeed == nil {
		err = errors.New("未查询到相关种子记录")
		return
	}
	toBeHappyOrchardSeed, err := happyOrchardSeedDb.GetHappyOrchardSeed(toBeSeedId)
	if err != nil {
		return
	}
	if toBeHappyOrchardSeed == nil {
		err = errors.New("未查询到相关种子记录")
		return
	}
	if nowHappyOrchardSeed.WaterNums > toBeHappyOrchardSeed.WaterNums {
		err = errors.New("不支持更大水滴数种子替换成小水滴数种子")
		return
	}

	//5、修改 happy_orchard_user_seed_record 记录 &&  happy_orchard_user_waters 记录
	record.SeedId = toBeSeedId
	happyOrchardUserWaters.ReplaceSeedNums--
	happyOrchardUserSeedRecordDb := implement.NewHappyOrchardUserSeedRecordDb(engine)
	_, err = happyOrchardUserSeedRecordDb.UpdateHappyOrchardUserSeedRecord(&record, "seed_id")
	if err != nil {
		return
	}
	_, err = happyOrchardUserWatersDb.UpdateHappyOrchardUserWatersBySess(uid, happyOrchardUserWaters, "replace_seed_nums")
	if err != nil {
		return err
	}
	return
}
