package one_circles

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule/one_circles/enum"
	md2 "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule/one_circles/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule/one_circles/svc"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/shopspring/decimal"
	"time"
	"xorm.io/xorm"
)

// AutoAdjustPrice 自动调整价格
func AutoAdjustPrice(engine *xorm.Engine, masterId string) (err error) {
	now := time.Now()
	fmt.Println(now.Hour())

	//1、查找 `one_circles_green_energy_basic_setting` 基础设置
	oneCirclesGreenEnergyBasicSetting, cb, err := svc.GetPrice(engine, masterId)
	if err != nil {
		return
	}
	if cb != nil {
		defer cb() // 释放锁
	}

	var priceSettingStruct *md2.PriceSettingStruct
	err = json.Unmarshal([]byte(oneCirclesGreenEnergyBasicSetting.PriceSetting), &priceSettingStruct)
	if err != nil {
		return
	}
	if priceSettingStruct.PriceHigherThanValue == "" || priceSettingStruct.MarketplaceMerchantsNumsExchangeMarketplaceMerchantsFundValue == "" || priceSettingStruct.PriceBelowValue == "" || priceSettingStruct.MarketplaceMerchantsFundExchangeMarketplaceMerchantsNumsValue == "" {
		err = errors.New("价格设置未完全！")
		return
	}

	session := engine.NewSession()
	defer func() {
		session.Close()
		if err := recover(); err != nil {
			_ = zhios_order_relate_logx.Error(err)
		}
	}()
	session.Begin()

	priceHigherThanValue, _ := decimal.NewFromString(priceSettingStruct.PriceHigherThanValue)
	marketplaceMerchantsNumsExchangeMarketplaceMerchantsFundValue, _ := decimal.NewFromString(priceSettingStruct.MarketplaceMerchantsNumsExchangeMarketplaceMerchantsFundValue)
	priceBelowValue, _ := decimal.NewFromString(priceSettingStruct.PriceBelowValue)
	marketplaceMerchantsFundExchangeMarketplaceMerchantsNumsValue, _ := decimal.NewFromString(priceSettingStruct.MarketplaceMerchantsFundExchangeMarketplaceMerchantsNumsValue)
	nowPriceValue, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.NowPrice)
	if nowPriceValue.GreaterThan(priceHigherThanValue) {
		//当价格涨到设置某个价格时，自动执行  市商数量 兑换 市商资金 (降价公式)
		err1, amount, _, afterPriceValue := NewCalcPriceReductionFormula(marketplaceMerchantsNumsExchangeMarketplaceMerchantsFundValue.String(), oneCirclesGreenEnergyBasicSetting)
		if err1 != nil {
			fmt.Println("err1111:::", err1)
			_ = session.Rollback()
			return err1
		}

		// 市商数量 减少、原始数量 增加、原始资金 减少、市商资金 增加
		err = DealAvailableGreenEnergyCoin(session, int(enum.MarketplaceMerchantNumsAutoExchangeMarketplaceMerchantFunds), zhios_order_relate_utils.StrToFloat64(marketplaceMerchantsNumsExchangeMarketplaceMerchantsFundValue.String()), zhios_order_relate_utils.StrToFloat64(amount), enum.MarketplaceMerchantNumsAutoExchangeMarketplaceMerchantFunds.String(), oneCirclesGreenEnergyBasicSetting, afterPriceValue)
		if err != nil {
			fmt.Println("err:::::22222", err)
			_ = session.Rollback()
			return err
		}
	}
	if nowPriceValue.LessThan(priceBelowValue) {
		//当价格降到设置某个价格时，自动执行  市商资金 兑换 市商数量 (涨价公式)
		err1, greenEnergy, _, afterPriceValue := NewCalcPriceIncreaseFormula(marketplaceMerchantsFundExchangeMarketplaceMerchantsNumsValue.String(), oneCirclesGreenEnergyBasicSetting)
		if err1 != nil {
			fmt.Println("err33333:::", err1)
			_ = session.Rollback()
			return err1
		}
		// 市商数量 增加、原始数量 减少、原始资金 增加、市商资金 减少
		err = DealAvailableGreenEnergyCoin(session, int(enum.MarketplaceMerchantFundsAutoExchangeMarketplaceMerchantNums), zhios_order_relate_utils.StrToFloat64(greenEnergy), zhios_order_relate_utils.StrToFloat64(marketplaceMerchantsFundExchangeMarketplaceMerchantsNumsValue.String()), enum.MarketplaceMerchantFundsAutoExchangeMarketplaceMerchantNums.String(), oneCirclesGreenEnergyBasicSetting, afterPriceValue)
		if err != nil {
			fmt.Println("err:::::44444", err)
			_ = session.Rollback()
			return err
		}
	}

	err = session.Commit()
	if err != nil {
		_ = session.Rollback()
		return errors.New("事务提交失败")
	}
	return
}
