package one_circles

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	md2 "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule/one_circles/md"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	"encoding/json"
	"errors"
	"github.com/shopspring/decimal"
	"time"
	"xorm.io/xorm"
)

// CalcPriceIncreaseFormula 计算涨价公式(【用户资金 ÷（用户资金+原始资金）÷ 原始数量】 = 用户获得绿色能量个数)
func CalcPriceIncreaseFormula(engine *xorm.Engine, userAmountValue string) (err error, values, nowPriceValue, afterPriceValue string) {
	//1、查找 `one_circles_green_energy_basic_setting` 基础设置
	oneCirclesGreenEnergyBasicSetting, err := db.OneCirclesGreenEnergyBasicSettingGetOneByParams(engine, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		return
	}
	userAmount, _ := decimal.NewFromString(userAmountValue)                                                  //用户资金
	nowPrice, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.NowPrice)                         //当前价格
	originalQuantityNums, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.OriginalQuantityNums) //原始数量
	originalQuantityFunds, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.OriginalFunds)       //原始资金
	afterPrice := (userAmount.Add(originalQuantityFunds)).Div(originalQuantityNums).Truncate(8)              //涨价后的价格

	values = userAmount.Div(afterPrice).Truncate(8).String() //用户得到绿色能量个数
	nowPriceValue = nowPrice.String()
	afterPriceValue = afterPrice.String()
	return
}

// NewCalcPriceIncreaseFormula 计算涨价公式(【用户资金 ÷（用户资金+原始资金）÷ 原始数量】 = 用户获得绿色能量个数)
func NewCalcPriceIncreaseFormula(userAmountValue string, oneCirclesGreenEnergyBasicSetting *model.OneCirclesGreenEnergyBasicSetting) (err error, values, nowPriceValue, afterPriceValue string) {
	userAmount, _ := decimal.NewFromString(userAmountValue)                                                  //用户资金
	nowPrice, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.NowPrice)                         //当前价格
	originalQuantityNums, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.OriginalQuantityNums) //原始数量
	originalQuantityFunds, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.OriginalFunds)       //原始资金
	afterPrice := (userAmount.Add(originalQuantityFunds)).Div(originalQuantityNums).Truncate(8)              //涨价后的价格

	values = userAmount.Div(afterPrice).Truncate(8).String() //用户得到绿色能量个数
	nowPriceValue = nowPrice.String()
	afterPriceValue = afterPrice.String()
	zhios_order_relate_utils.FilePutContents("virtual_coin_not", zhios_order_relate_utils.SerializeStr(map[string]interface{}{
		"before_price": nowPriceValue,
		"after_price":  afterPriceValue,
		"user_amount":  userAmountValue,
	}))
	return
}

// CalcPriceReductionFormula 计算降价公式(【用户需兑换绿能色量数量*｛原始资金÷（用户需兑换绿色数量+原始数量｝*（1 - 扣比例50% ~ 23%） = 用户获得钱)
func CalcPriceReductionFormula(engine *xorm.Engine, userExchangeNumsValue string, levelId string, uid int) (err error, values, greenEnergy, greenEnergyFee, nowPriceValue, afterPriceValue string) {
	//1、查找 `one_circles_green_energy_basic_setting` 基础设置
	oneCirclesGreenEnergyBasicSetting, err := db.OneCirclesGreenEnergyBasicSettingGetOneByParams(engine, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		return
	}
	var vipEquitySetting []*md2.VipEquitySettingStruct
	err = json.Unmarshal([]byte(oneCirclesGreenEnergyBasicSetting.VipEquitySetting), &vipEquitySetting)
	if err != nil {
		return
	}
	var exchangeAccountBalanceFeeValue string
	for _, v := range vipEquitySetting {
		if v.VipLevelId == levelId {
			exchangeAccountBalanceFeeValue = v.ExchangeAccountBalanceFee
		}
	}
	if exchangeAccountBalanceFeeValue == "" {
		err = errors.New("未查询到当前会员等级兑换余额手续费")
		return
	}
	decimalRate := decimal.NewFromInt(100)                                                //百分比
	exchangeAccountBalanceFee, _ := decimal.NewFromString(exchangeAccountBalanceFeeValue) //兑换手续费
	//TODO::统计当前有多少直推用户昨天签到了
	userRelates, err := db.DbsUserRelateByParentUid(engine, uid, 1)
	if err != nil {
		return
	}
	var userRelatesUids []int
	if userRelates != nil {
		for _, userRelate := range *userRelates {
			userRelatesUids = append(userRelatesUids, userRelate.Uid)
		}
	}
	if len(userRelatesUids) > 0 {
		now := time.Now()
		startDate := now.AddDate(0, 0, -1).Format("2006-01-02") + " 00:00:00"
		endDate := now.Format("2006-01-02") + " 00:00:00"
		var oneCirclesGreenEnergySignIns []*model.OneCirclesGreenEnergySignIn
		err = engine.Where("start_time >= ?", startDate).And("start_time <=?", endDate).In("uid", userRelatesUids).And("is_completed =?", 0).Find(&oneCirclesGreenEnergySignIns)
		if err != nil {
			return
		}

		decimalRateV2 := decimal.NewFromInt(10) //百分比
		if len(oneCirclesGreenEnergySignIns) > 0 {
			decimalBase := decimal.NewFromInt(1) //基数
			userRelatesUidsValue := decimal.NewFromInt(int64(len(userRelatesUids)))
			oneCirclesGreenEnergySignInsValue := decimal.NewFromInt(int64(len(oneCirclesGreenEnergySignIns)))
			exchangeAccountBalanceFee = exchangeAccountBalanceFee.Add(decimalBase.Sub(oneCirclesGreenEnergySignInsValue.Div(userRelatesUidsValue)).Mul(decimalRateV2))
		} else {
			exchangeAccountBalanceFee = exchangeAccountBalanceFee.Add(decimalRateV2)
		}
	}

	userExchangeNums, _ := decimal.NewFromString(userExchangeNumsValue)                                      //用户兑换绿色能量
	originalQuantityNums, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.OriginalQuantityNums) //原始数量
	nowPrice, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.NowPrice)                         //当前价格
	originalQuantityFunds, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.OriginalFunds)       //原始资金
	afterPrice := originalQuantityFunds.Div(userExchangeNums.Add(originalQuantityNums))                      //降价后的价格

	greenEnergyValues := userExchangeNums.Mul(afterPrice)                                     //绿色能量个数
	greenEnergyFeeValues := greenEnergyValues.Mul(exchangeAccountBalanceFee.Div(decimalRate)) //绿色能量个数扣除手续费价值

	values = greenEnergyValues.Sub(greenEnergyFeeValues).Truncate(8).String()                              //用户实际得到的钱
	greenEnergy = greenEnergyValues.Truncate(8).String()                                                   //原本兑换到的钱
	greenEnergyFee = userExchangeNums.Mul(exchangeAccountBalanceFee.Div(decimalRate)).Truncate(8).String() //绿色能量手续费
	nowPriceValue = nowPrice.String()
	afterPriceValue = afterPrice.String()

	zhios_order_relate_utils.FilePutContents("virtual_coin_not", zhios_order_relate_utils.SerializeStr(map[string]interface{}{
		"before_price": nowPriceValue,
		"after_price":  afterPriceValue,
		"user_amount":  userExchangeNumsValue,
	}))
	return
}

// CalcPriceReductionFormula 计算降价公式(【用户需兑换绿能色量数量*｛原始资金÷（用户需兑换绿色数量+原始数量｝*（1 - 扣比例50% ~ 23%） = 用户获得钱)
//func CalcPriceReductionFormula(engine *xorm.Engine, userExchangeNumsValue string, levelId string) (err error, values, greenEnergy, greenEnergyFee, nowPriceValue, afterPriceValue string) {
//	//1、查找 `one_circles_green_energy_basic_setting` 基础设置
//	oneCirclesGreenEnergyBasicSetting, err := db.OneCirclesGreenEnergyBasicSettingGetOneByParams(engine, map[string]interface{}{
//		"key":   "is_open",
//		"value": 1,
//	})
//	if err != nil {
//		return
//	}
//	var vipEquitySetting []*md2.VipEquitySettingStruct
//	err = json.Unmarshal([]byte(oneCirclesGreenEnergyBasicSetting.VipEquitySetting), &vipEquitySetting)
//	if err != nil {
//		return
//	}
//	var exchangeAccountBalanceFeeValue string
//	for _, v := range vipEquitySetting {
//		if v.VipLevelId == levelId {
//			exchangeAccountBalanceFeeValue = v.ExchangeAccountBalanceFee
//		}
//	}
//	if exchangeAccountBalanceFeeValue == "" {
//		err = errors.New("未查询到当前会员等级兑换余额手续费")
//		return
//	}
//	decimalRate := decimal.NewFromInt(100)                                                                   //百分比
//	exchangeAccountBalanceFee, _ := decimal.NewFromString(exchangeAccountBalanceFeeValue)                    //兑换手续费
//	userExchangeNums, _ := decimal.NewFromString(userExchangeNumsValue)                                      //用户兑换绿色能量
//	originalQuantityNums, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.OriginalQuantityNums) //原始数量
//	nowPrice, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.NowPrice)                         //当前价格
//	originalQuantityFunds, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.OriginalFunds)       //原始资金
//	afterPrice := originalQuantityFunds.Div(userExchangeNums.Add(originalQuantityNums))                      //降价后的价格
//
//	greenEnergyValues := userExchangeNums.Mul(afterPrice)                                     //绿色能量个数
//	greenEnergyFeeValues := greenEnergyValues.Mul(exchangeAccountBalanceFee.Div(decimalRate)) //绿色能量个数扣除手续费价值
//
//	values = greenEnergyValues.Sub(greenEnergyFeeValues).Truncate(8).String()                              //用户实际得到的钱
//	greenEnergy = greenEnergyValues.Truncate(8).String()                                                   //原本兑换到的钱
//	greenEnergyFee = userExchangeNums.Mul(exchangeAccountBalanceFee.Div(decimalRate)).Truncate(8).String() //绿色能量手续费
//	nowPriceValue = nowPrice.String()
//	afterPriceValue = afterPrice.String()
//	return
//}

// NewCalcPriceReductionFormula 计算降价公式(【用户需兑换绿能色量数量*｛原始资金÷（用户需兑换绿色数量+原始数量｝*（1 - 扣比例50% ~ 23%） = 用户获得钱)
func NewCalcPriceReductionFormula(userExchangeNumsValue string, oneCirclesGreenEnergyBasicSetting *model.OneCirclesGreenEnergyBasicSetting) (err error, greenEnergy, nowPriceValue, afterPriceValue string) {
	userExchangeNums, _ := decimal.NewFromString(userExchangeNumsValue)                                      //用户兑换绿色能量
	originalQuantityNums, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.OriginalQuantityNums) //原始数量
	nowPrice, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.NowPrice)                         //当前价格
	originalQuantityFunds, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.OriginalFunds)       //原始资金
	afterPrice := originalQuantityFunds.Div(userExchangeNums.Add(originalQuantityNums))                      //降价后的价格

	greenEnergyValues := userExchangeNums.Mul(afterPrice) //用户获得的钱

	greenEnergy = greenEnergyValues.Truncate(8).String()
	nowPriceValue = nowPrice.String()
	afterPriceValue = afterPrice.String()
	return
}

// JudgeUserIsCanBuyWelfareOrders 判断用户是否可以购买福利订单
func JudgeUserIsCanBuyWelfareOrders(engine *xorm.Engine, uid int) (err error) {
	now := time.Now()
	//1、查找 `one_circles_green_energy_basic_setting` 基础设置
	var oneCirclesGreenEnergyBasicSetting model.OneCirclesGreenEnergyBasicSetting
	has, err := engine.Where("is_open = 1").Get(&oneCirclesGreenEnergyBasicSetting)
	if err != nil {
		return
	}
	if !has {
		return
	}
	var welfareOrdersLimit *md2.WelfareOrdersLimit
	err = json.Unmarshal([]byte(oneCirclesGreenEnergyBasicSetting.WelfareOrdersLimit), &welfareOrdersLimit)
	if err != nil {
		return
	}

	//2、计算当前用户连续日活
	startDate := now.AddDate(0, 0, -7).Format("2006-01-02")
	endDate := now.Format("2006-01-02")
	err, days, isContinuousDailyActivity := CalcUserContinuousDailyActivityDays(engine, "", uid, startDate, endDate)
	if err != nil {
		return
	}
	if !isContinuousDailyActivity {
		err = errors.New("未连续日活不可购买")
		return
	}
	if days < zhios_order_relate_utils.StrToInt(welfareOrdersLimit.ContinuousDailyActivityNums) {
		err = errors.New("连续日活天数不够")
		return
	}

	//3、计算直推人数
	userRelates, err := db.DbsUserRelateByParentUid(engine, uid, 1)
	if err != nil {
		return
	}
	if len(*userRelates) < zhios_order_relate_utils.StrToInt(welfareOrdersLimit.DirectPushLimit.DirectPushUserNums) {
		err = errors.New("直推人数不够")
	}

	return
}
